<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
    xmlns:ocil="http://www.mitre.org/ocil/1.1">

    <xsl:output method="html"/>

    <xsl:key name="questionnaire_id"
        match="ocil:ocil/ocil:questionnaire" use="@id"/>
    <xsl:key name="question_test_action_id" match="ocil:ocil/ocil:boolean_question_test_action|ocil:choice_question_test_action|ocil:numeric_question_test_action|ocil:string_question_test_action"
        use="@id"/>
    <xsl:key name="question_id" match="ocil:ocil/ocil:boolean_question|ocil:choice_question|ocil:numeric_question|ocil:string_question"
        use="@id"/>
    
    <xsl:template match="/">
        <html>
            <head>
                <link href="results.css" type="text/css" rel="stylesheet"/>
                <title>ocil Document Results</title>
            </head>
            <body>
                <xsl:apply-templates select="ocil:ocil" />
                <xsl:apply-templates select="ocil:ocil/ocil:results" />
            </body>
        </html>
    </xsl:template>

    <xsl:template match="ocil:ocil">
        <h1>OCIL Document</h1>
        <h3>Generator</h3>
        <table>
            <tr>
                <th>Field Name</th>
                <th>Value</th>
            </tr>
            <xsl:for-each select="ocil:generator/*">
                <tr>
                    <td>
                        <xsl:value-of select="local-name()"/>
                    </td>
                    <td>
                        <xsl:value-of select="."/>
                    </td>
                </tr>
            </xsl:for-each>
        </table>
    </xsl:template>
    
    <xsl:template match="ocil:results">
        <h3>Questionnaire Results</h3>
        <table>
            <tr>
                <th>Questionnaire Id</th>
                <th>Results</th>
                <th>Title</th>
                <th>Description</th>
            </tr>
            <xsl:apply-templates select="ocil:questionnaire_result" />    
        </table>        
        <h3>Question Test Action Results</h3>
        <table>
            <tr>
                <th>Question Test Action Id</th>
                <th>Results</th>
                <th>Title</th>
                <th>Description</th>
            </tr>
            <xsl:apply-templates select="ocil:test_action_result" />    
        </table>
        <h3>Question Answers or Results</h3>
        <table>
            <tr>
                <th>Question Id</th>
                <th>Text</th>
                <th>Answer/Result</th>
            </tr>
            <xsl:apply-templates select="ocil:boolean_question_result" />    
            <xsl:apply-templates select="ocil:choice_question_result" />    
            <xsl:apply-templates select="ocil:numeric_question_result" />    
            <xsl:apply-templates select="ocil:string_question_result" />    
        </table>
    </xsl:template>
    
    <xsl:template match="ocil:questionnaire_result">
        <xsl:variable name="qnid" select="@questionnaire_ref"/>
        <xsl:variable name="qnres" select="@result"/>
        <xsl:variable name="qn" select="key('questionnaire_id', $qnid)" />
        <xsl:variable name="qntitle" select="$qn/ocil:title"/>
        <xsl:variable name="qndesc" select="$qn/ocil:description"/>
        <tr>
            <td><xsl:value-of select="$qnid"/></td>
            <td><xsl:value-of select="$qnres"/></td>                    
            <td>
                <xsl:choose>
                    <xsl:when test="string-length($qntitle)=0">
                        No Title.
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="$qntitle"/>                        
                    </xsl:otherwise>
                </xsl:choose>
            </td>
            <td>
                <xsl:choose>
                    <xsl:when test="string-length($qndesc)=0">
                        No Description.
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="$qndesc"/>                        
                    </xsl:otherwise>
                </xsl:choose>
            </td>
        </tr>
    </xsl:template>

    <xsl:template match="ocil:test_action_result">
        <xsl:variable name="taid" select="@test_action_ref"/>
        <xsl:variable name="tares" select="@result"/>
        <xsl:variable name="ta" select="key('question_test_action_id', $taid)" />
        <xsl:variable name="tatitle" select="$ta/ocil:title"/>
        <xsl:variable name="tadesc" select="$ta/ocil:description"/>
        <tr>
            <td><xsl:value-of select="$taid"/></td>
            <td><xsl:value-of select="$tares"/></td>                    
            <td>
                <xsl:choose>
                    <xsl:when test="string-length($tatitle)=0">
                        No Title.
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="$tatitle"/>                        
                    </xsl:otherwise>
                </xsl:choose>
            </td>
            <td>
                <xsl:choose>
                    <xsl:when test="string-length($tadesc)=0">
                        No Description.
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="$tadesc"/>                        
                    </xsl:otherwise>
                </xsl:choose>
            </td>
         </tr>
    </xsl:template>
    
    <xsl:template match="ocil:boolean_question_result|ocil:numeric_question_result|ocil:string_question_result">
        <xsl:variable name="question_id" select="@question_ref"/>
        <xsl:variable name="question_answer" select="ocil:answer"/>
        <xsl:variable name="question_response" select="@response"/>
        <xsl:variable name="question" select="key('question_id', $question_id)" />
        <xsl:variable name="question_text" select="$question/ocil:question_text"/>
        <tr>
            <td><xsl:value-of select="$question_id"/></td>
            <td><xsl:value-of select="$question_text"/></td>  
            <td>
                <xsl:if test="$question_response='ANSWERED'">
                    <xsl:value-of select="$question_answer"/>                    
                </xsl:if>
                <xsl:if test="$question_response!='ANSWERED'">
                    <xsl:value-of select="$question_response"/>                    
                </xsl:if>
            </td>  
        </tr>
    </xsl:template>
    
    <xsl:template match="ocil:choice_question_result">
        <xsl:variable name="question_id" select="@question_ref"/>
        <xsl:variable name="question_choice_ref" select="ocil:answer/@choice_ref"/>
        <xsl:variable name="question_response" select="@response"/>
        <xsl:variable name="question" select="key('question_id', $question_id)" />
        <xsl:variable name="question_answer" select="$question/ocil:choice[@id=$question_choice_ref]"/>
        <xsl:variable name="question_text" select="$question/ocil:question_text"/>
        <tr>
            <td><xsl:value-of select="$question_id"/></td>
            <td><xsl:value-of select="$question_text"/></td>  
            <td>
                <xsl:if test="$question_response='PASS'">
                    <xsl:value-of select="$question_answer"/>                    
                </xsl:if>
                <xsl:if test="$question_response!='PASS'">
                    <xsl:value-of select="$question_response"/>                    
                </xsl:if>
            </td>  
        </tr>
    </xsl:template>

</xsl:stylesheet>